#pragma once

#include "const.h"
#include <vector>
#include <atlstr.h>
#include "util.h"

/**
* @class ProgramInformation
* InstallComponent Ѱֱ  InstallProgramInformationList 
* ϴ ProgramInformation Ŭ.
*
* @author : DaeMyung.Kang
*/

enum InstallProcessType{
	Error = 0,
	InstallType,
	UpgradeType,
	UninstallType,
};

struct ProgramInformation
{
	ProgramInformation()
	{
		execute_option	= TRUE;
		install_process_type = Error;
		url[0]			= 0;
		path[0]			= 0;
	}

	TCHAR		guid[GUID_SIZE];
	TCHAR		url[URL_SIZE];
	TCHAR		path[PATH_SIZE];
	int			execute_option;
	InstallProcessType	install_process_type;
};

struct ERRORINFO
{
	enum NAVERAPP_QUERY_ERROR
	{
		NQUERY_ERROR_NONE = 0,				//  
		NQUERY_ERROR_WRONGQUERY,			// ߸  
		NQUERY_ERROR_NOTENOUGHBUF,			// ۰ 
		NQUERY_ERROR_NETWORK,				// Ʈũ 
		NQUERY_ERROR_ALREADYUPGRADERRUN,	// ̹ ׷̴  Ǿ
		NQUERY_ERROR_NOUPGRADERFILE,		// ׷̴  
		//    ߰  
		NQUERY_ERROR_DISKEMPTY,
		NQUERY_ERROR_ETC = 1000,			// Ÿ , ޽ʿ
	};

	ERRORINFO()
	{
		code	= 0;
		message = NULL;
	}

	~ERRORINFO()
	{
		if( message )
		{
			delete [] message;
			message = NULL;
		}
	}

	int		code;
	TCHAR*	message;
};

class CALLBACK_FINISHED_INFO
{
public:
	CALLBACK_FINISHED_INFO()
	{
		guid[0] = 0;
		info = NULL;
	}

	~CALLBACK_FINISHED_INFO()
	{
		REMOVE_OBJECT(info);
	}

	void SetErrorInfo( ERRORINFO *info )
	{
		this->info = info;
// 		this->info.code = info->code;
// 		this->info.message = new TCHAR[lstrlen(info->message)+1];
// 		lstrcpy( this->info.message, info->message );
	}

	void SetGuid(const TCHAR *guid)
	{
		Checked::tcscpy_s( this->guid, 128, guid );
	}

	void SetFinishedStatus( CALLBACK_FINISHED_STATUS status )
	{
		this->status = status;
	}

	const TCHAR *GetGuid()
	{
		return guid;
	}

	CALLBACK_FINISHED_STATUS GetFinishedStatus()
	{
		return status;
	}

	ERRORINFO *GetErrorInfo()
	{
		return info;
	}

private:
	ERRORINFO						*info;
	TCHAR							guid[128];
	CALLBACK_FINISHED_STATUS		status;
};

typedef std::vector<ProgramInformation> InstallProgramInformationVector;

class IPrepareCallbackBase
{
public:
	virtual CALLBACK_RETURN OnPrepareStart( const TCHAR* guid ) = 0;

	/** 
	* @fn CALLBACK_RETURN OnProgress( const TCHAR* guid, 
	*								  CALLBACK_PROGRESS_SIZE currentSize,
	*								  CALLBACK_PROGRESS_SIZE maxSize,
	*								  CALLBACK_PROGRESS_STATUS status )
	* @brief  ٿε ÿ    ˷ִ Լ̴.
	* @param
	*			guid		:   α׷ ˷ִ α׷ GUID
	*			currentSize :  ٿε  
	*			maxSize		:  ִ 
	*			status		: , PENDING  ˷ش.
	* @return 
	*			CALLBACK_RETURN_CANCEL		:  ۾ Ѵ.
	*			CALLBACK_RETURN_CONTINUE	:  ۾ Ѵ.
	*/
	virtual CALLBACK_RETURN OnProgress( const TCHAR* guid, 
		CALLBACK_PROGRESS_SIZE currentSize,
		CALLBACK_PROGRESS_SIZE maxSize,
		CALLBACK_PROGRESS_STATUS status ) = 0;

	/** 
	* @fn CALLBACK_RETURN OnPrepareFinished( const TCHAR* guid, 
	*										 CALLBACK_FINISHED_STATUS status )
	* @brief Prepare ۾   ˷ִ Լ̴.
	* @param
	*			guid		:   α׷ ˷ִ α׷ GUID
	*			status		:  ¸ ˷ش.(  , ,  )
	* @return 
	*			CALLBACK_RETURN_CANCEL		: Install ۾  ʴ´.
	*			CALLBACK_RETURN_CONTINUE	: Install ۾ Ѵ.
	*/
	virtual CALLBACK_RETURN OnPrepareFinished( const TCHAR* guid,
		CALLBACK_FINISHED_STATUS status, ERRORINFO *info ) = 0;
};

class IInstallCallbackBase
{
public:
	virtual CALLBACK_RETURN OnInstallStart( const TCHAR* guid ) = 0;

	/** 
	* @fn CALLBACK_RETURN OnInstallFinished( const TCHAR* guid, 
	*										 CALLBACK_FINISHED_STATUS status )
	* @brief Install ۾   ˷ִ Callback
	* @param
	*			guid		:   α׷ ˷ִ α׷ GUID
	*			status		:  ¸ ˷ش.(  , ,  )
	* @return 
	*			CALLBACK_RETURN_CANCEL		:  ۾ Ѵ.
	*			CALLBACK_RETURN_CONTINUE	:  ۾ Ѵ.
	*/
	virtual CALLBACK_RETURN OnInstallFinished( const TCHAR* guid,
		CALLBACK_FINISHED_STATUS status, ERRORINFO *info ) = 0;
};

/**
* @class IInstallComponentCallback
* InstallComponent Ѱִ Callback ޱ  Ŭ
*
* @author : DaeMyung.Kang
*/
class IInstallControllerCallback : public IPrepareCallbackBase, public IInstallCallbackBase
{
public:
	IInstallControllerCallback(){}
	virtual ~IInstallControllerCallback(){}
};


class IInstallController
{
public:
	virtual ~IInstallController(){}

	virtual BOOL Install( InstallProgramInformationVector infoArray, 
		IInstallControllerCallback *InstallControllerCallback ) = 0;
	virtual BOOL Remove( InstallProgramInformationVector infoArray, 
		IInstallControllerCallback *InstallControllerCallback ) = 0;
	virtual BOOL SetPending( const TCHAR *guid, BOOL flag ) = 0;
	virtual BOOL Cancel(const TCHAR *guid) = 0;
	virtual BOOL Create() = 0;
};

IInstallController *CreateInstallController();
